print("Loading EJIntegration!")

local Dialog = LibStub("LibDialog-1.0")

local desireOptions = {"BiS", "Upgrade", "Off-spec", "Not Needed"}
local desireColors = {{r=0, g=1, b=0}, {r=1, g=1, b=0}, {r=1, g=0.5, b=0}, {r=1, g=0, b=0}}


Dialog:Register("BossListBlacklistConfirmationDialog", {
  width = 400,
  buttons = {
    {
      text = "Yes",
      on_click = function(self)
                    BossList.AddToBlackList(self.data.name)
                 end
    },
    {
      text = "No"
    }
  },
  on_show = function(self)
              self.text:SetText("You are about to add player |cFF3BFFF2" .. self.data.name .. "|r to your blacklist. By doing so, all entries of this player will be |cFFFF3B3Eremoved|r from BossList, and no entries from this player will be added again until this player has been removed from the blacklist through the interface options. Are you sure this is what you want?")
            end,
  hide_on_escape = true,
  
})

local function pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end

function BossList.DifficultyChanged(newDifficulty)
  -- If the new difficulty is not supported by BossList (== 0), disable the tab
  if newDifficulty == 0 then
    BossListJournalTab:Disable()
    
    -- Click the first tab if the BossList tab was selected
    if EncounterJournal.encounter.info.tab == 4 then
      EncounterJournalEncounterFrameInfoBossTab:Click()
    end
  else
    BossListJournalTab:Enable()
    -- If the BossList tab was being shown, make sure it is updated with the new difficulty
    if EncounterJournal.encounter.info.tab == 4 then
      BossListJournalTab:Click()
    end
  end
end

function BossList.BlacklistPlayer(name)
  Dialog:Spawn("BossListBlacklistConfirmationDialog", {name = name})
end

local function CreateDesireLists(frame)
  for idx, desire in pairs(desireOptions) do
    local desireList = CreateFrame("Frame", "BossListDesire" .. tostring(idx), frame, "BossListBossDesire")
    desireList.title:SetText(desire)
    desireList.title:SetTextColor(desireColors[idx].r, desireColors[idx].g, desireColors[idx].b)
    desireList.entries = {}
    
    if idx > 1 then
      desireList:SetPoint("TOP", "BossListDesire" .. tostring(idx - 1) .. "ListBG", "BOTTOM", 0, -5)
    end
  end
end

local createdDesireLists = false
function BossList.DisplayEntries()
  -- If it is the first time, setup the desire lists
  if not(createdDesireLists) then
    CreateDesireLists(BossListScrollFrameScrollChild)
    createdDesireLists = true
  end
  
  local instanceID = EncounterJournal.instanceID
  local encounterID = EncounterJournal.encounterID
  local difficultyID = BossList.difficulty
  
  -- Check if there are any entries for this instance-encounter-difficulty combination to show
  if BossList_entries[instanceID] == nil or 
     BossList_entries[instanceID][difficultyID] == nil or
     BossList_entries[instanceID][difficultyID][encounterID] == nil then
    
    -- Hide all entries
    for desireIdx, desire in pairs(desireOptions) do
      local desireList = _G["BossListDesire" .. desireIdx]
      local entries = desireList.entries
      for i = 1, #entries do
        entries[i]:Hide()
      end
      
      -- Retract the background
      desireList.listBG:SetPoint("BOTTOM", desireList, "BOTTOM")
    end
    
    return
  end
    
  local data = BossList_entries[instanceID][difficultyID][encounterID]
  
  -- Keep track of players that are already being shown, so they do not appear under multiple desires
  local printedPlayers = {}
  for desireIdx, desire in pairs(desireOptions) do
    local desireList = _G["BossListDesire" .. desireIdx]
    local entries = desireList.entries
    
    -- Find all players that have the current desire for this encounter, by checking each item
    local presentPlayers = {}
    for item, desires in pairs(data) do
      local players = desires[desireIdx] or {}
      
      for player, _ in pairs(players) do
        if not(printedPlayers[player]) then
          presentPlayers[player] = true
        end
      end
    end
    
    -- Create entries for each of the players found
    local counter = 1
    for player, _ in pairsByKeys(presentPlayers) do
      local entry;
      if entries[counter] == nil then -- Create a new entry if needed
        entry = CreateFrame("Frame", "BossListEntryFrame", desireList, "BossListEntry")
        entry.name:SetTextColor(desireColors[desireIdx].r,desireColors[desireIdx].g, desireColors[desireIdx].b)
        
        -- Put it below the previous entry
        if counter > 1 then
          entry:SetPoint("TOP", entries[counter - 1], "BOTTOM")
        else
          entry:SetPoint("TOP", desireList, "BOTTOM", 0, -2)
        end
        
        entries[counter] = entry
      else
        entry = entries[counter]
        entry:Show()
      end
      
      entry.name:SetText(player)
      
      counter = counter + 1
      printedPlayers[player] = true
    end
    
    -- Hide unused entries
    for i = counter, #entries do
      entries[i]:Hide()
    end
    
    -- Extend the background to the last entry, if any were added
    if counter > 1 then
      desireList.listBG:SetPoint("BOTTOM", entries[counter-1], "BOTTOM")
    else
      desireList.listBG:SetPoint("BOTTOM", desireList, "BOTTOM")
    end
  end
end

-- Enable/Disable the BossList tab depending on whether an instance is selected or not
hooksecurefunc("EncounterJournal_DisplayEncounter", function()
  if BossList.difficulty ~= 0 then
    BossListJournalTab:Enable()
  end
end)

hooksecurefunc("EncounterJournal_DisplayInstance", function()
  BossListJournalTab:Disable()
end)

-- Hook the hiding of the tab into the other tabs of the encounter journal
hooksecurefunc("EncounterJournal_TabClicked", function()
  BossListScrollFrame:Hide()
end)

-- The loottab button strangely enough doesn't call the TabClicked function, so hook it directly
EncounterJournalEncounterFrameInfoLootTab:HookScript("OnClick", function()
  BossListScrollFrame:Hide()
end)

-- When DisplayEncounter is called, having the BossListTab selected causes an out of bounds error. Temporarily change the index when it is called.
local Old_EncounterJournal_DisplayEncounter = EncounterJournal_DisplayEncounter

EncounterJournal_DisplayEncounter = function(encounterID, noButton)
  local changedTab = false
  if EncounterJournal.encounter.info.tab == 4 then
    EncounterJournal.encounter.info.tab = 1
    changedTab = true
  end  
  
  Old_EncounterJournal_DisplayEncounter(encounterID, noButton)
  
  if changedTab then
    BossListJournalTab:Click()
  end
end
